@@ -2,7 +2,7 @@ package Dancer::Session::Cookie;
 use strict;
 use warnings;
 # ABSTRACT: Encrypted cookie-based session backend for Dancer
-our $VERSION = '0.20'; # VERSION
+our $VERSION = '0.21'; # VERSION
 
 use base 'Dancer::Session::Abstract';
 
@@ -12,7 +12,7 @@ use String::CRC32;
 use Crypt::Rijndael;
 use Time::Duration::Parse;
 
-use Dancer ':syntax';
+use Dancer 1.3113 ':syntax'; # 1.3113 for on_reset_state and fixed after hook
 use Dancer::Cookie  ();
 use Dancer::Cookies ();
 use Storable        ();
@@ -25,6 +25,8 @@ my $STORE  = undef;
 # cache session here instead of flushing/reading from cookie all the time
 my $SESSION = undef;
 
+sub is_lazy { 1 }; # avoid calling flush needlessly
+
 sub init {
     my ($self) = @_;
 
@@ -109,20 +111,20 @@ sub destroy {
 hook 'after' => sub {
     my $response = shift;
 
-    # UGH! Awful hack because Dancer instantiates responses
-    # and headers too many times and locks out new cookies
-    $response->{_built_cookies} = 0;
-
     if ($SESSION) {
+        # UGH! Awful hack because Dancer instantiates responses
+        # and headers too many times and locks out new cookies
+        $response->{_built_cookies} = 0;
+
         my $c = Dancer::Cookie->new( $SESSION->_cookie_params );
         Dancer::Cookies->set_cookie_object( $c->name => $c );
-        undef $SESSION; # clear for next request
     }
 };
 
 # Make sure that the session is initially undefined for every request
-hook 'before' => sub {
-	undef $SESSION;
+hook 'on_reset_state' => sub {
+    my $is_forward = shift;
+    undef $SESSION unless $is_forward;
 };
 
 # modified from Dancer::Session::Abstract::write_session_id to add
@@ -180,8 +182,6 @@ sub _old_decrypt {
 
 1;
 
-__END__
-
 =pod
 
 =encoding utf-8
@@ -192,7 +192,7 @@ Dancer::Session::Cookie - Encrypted cookie-based session backend for Dancer
 
 =head1 VERSION
 
-version 0.20
+version 0.21
 
 =head1 SYNOPSIS
 
@@ -304,6 +304,10 @@ Michael G. Schwern <schwern@pobox.com>
 
 Neil Kirsopp <neil@broadbean.com>
 
+=item *
+
+Nick S. Knutov <nick@knutov.com>
+
 =back
 
 =head1 COPYRIGHT AND LICENSE
@@ -314,3 +318,8 @@ This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
 
 =cut
+
+__END__
+
+
+# vim: ts=4 sts=4 sw=4 et:
